@@ -22,12 +22,12 @@ from ast import (
 )
 
 from cdd.ast_utils import FALLBACK_TYP, maybe_type_comment, set_arg, set_value
-from cdd.pure_utils import deindent, tab
+from cdd.pure_utils import deindent, emit_separating_tabs, tab
 from cdd.tests.mocks.classes import (
     class_torch_nn_l1loss_docstring_str,
     tensorboard_doc_str_no_args_str,
 )
-from cdd.tests.mocks.docstrings import docstring_header_str
+from cdd.tests.mocks.docstrings import docstring_header_no_nl_str, docstring_header_str
 from cdd.tests.utils_for_tests import remove_args_from_docstring
 
 argparse_add_argument_ast = Expr(
@@ -51,31 +51,65 @@ argparse_add_argument_ast = Expr(
     )
 )
 
-argparse_doc_str = (
+_argparse_doc_str_tuple = (
     "Set CLI arguments\n",
     ":param argument_parser: argument parser",
     ":type argument_parser: ```ArgumentParser```\n",
 )
 
 _cli_doc_str = "\n{tab}".format(tab=tab).join(
-    argparse_doc_str
+    _argparse_doc_str_tuple
     + (
-        ":returns: argument_parser, Train and tests dataset splits.",
+        ":return: argument_parser, Train and tests dataset splits.",
         ":rtype: ```Tuple[ArgumentParser, Union[Tuple[tf.data.Dataset, tf.data.Dataset],"
         " Tuple[np.ndarray,\n{tab}np.ndarray]]]```".format(tab=tab),
     )
 )
-_cli_doc_expr = Expr(set_value("\n{tab}{}\n{tab}".format(_cli_doc_str, tab=tab)))
 
-_cli_doc_nosplit_str = "\n{tab}".format(tab=tab).join(
-    argparse_doc_str
-    + (
-        ":returns: argument_parser",
-        ":rtype: ```ArgumentParser```",
+_argparse_doc_tuple = (
+    _argparse_doc_str_tuple[0].rstrip("\n"),
+    "",
+    _argparse_doc_str_tuple[1],
+    _argparse_doc_str_tuple[2],
+    ":return: argument_parser, Train and tests dataset splits.",
+    ":rtype: ```Tuple[ArgumentParser, Union[Tuple[tf.data.Dataset, tf.data.Dataset], Tuple[np.ndarray,",
+    "np.ndarray]]]```",
+)
+
+_argparse_doc_str = "\n".join(_argparse_doc_tuple)
+
+_argparse_description_str = emit_separating_tabs(
+    "\n{tab}".format(tab=tab).join(
+        _argparse_doc_tuple[:-1]
+        + (
+            "{tab}{last_line_of_doc_str}".format(
+                tab=tab, last_line_of_doc_str=_argparse_doc_tuple[-1]
+            ),
+        )
     )
 )
+_argparse_doc_stripped_str = _argparse_doc_str.replace(
+    "{tab}\n{tab}".format(tab=tab), ""
+)
+
+_cli_doc_expr = Expr(set_value(_argparse_description_str))
+
+_cli_doc_nosplit_str = emit_separating_tabs(
+    "\n{tab}".format(tab=tab).join(
+        _argparse_doc_str_tuple
+        + (
+            ":return: argument_parser",
+            ":rtype: ```ArgumentParser```",
+        )
+    )
+).strip(" \n")
+
 _cli_doc_nosplit_expr = Expr(
-    set_value("\n{tab}{}\n{tab}".format(_cli_doc_nosplit_str, tab=tab))
+    set_value(
+        "\n{tab}{_cli_doc_nosplit_str}\n{tab}".format(
+            _cli_doc_nosplit_str=_cli_doc_nosplit_str, tab=tab
+        )
+    )
 )
 
 _argparse_add_arguments = (
@@ -190,7 +224,7 @@ _argparse_add_arguments = (
                 keyword(
                     arg="help",
                     value=set_value(
-                        "Convert to numpy ndarrays.",
+                        "Convert to numpy ndarrays",
                     ),
                     identifier=None,
                 ),
@@ -280,7 +314,7 @@ def set_cli_args(argument_parser):
         default="np",
     )
     argument_parser.add_argument(
-        "--as_numpy", type=bool, help="Convert to numpy ndarrays.",
+        "--as_numpy", type=bool, help="Convert to numpy ndarrays",
     )
     argument_parser.add_argument(
         "--data_loader_kwargs",
@@ -318,7 +352,7 @@ def set_cli_args(argument_parser):
         required=True,
         default='np',
     )
-    argument_parser.add_argument('--as_numpy', type=bool, help='Convert to numpy ndarrays.')
+    argument_parser.add_argument('--as_numpy', type=bool, help='Convert to numpy ndarrays')
     argument_parser.add_argument(
         '--data_loader_kwargs', type=loads, help='pass this as arguments to data_loader function'
     )
@@ -389,7 +423,7 @@ argparse_func_ast = fix_missing_locations(
                         Store(),
                     )
                 ],
-                value=set_value(docstring_header_str.replace("\n", "")),
+                value=set_value(docstring_header_no_nl_str),
                 expr=None,
                 **maybe_type_comment
             ),
@@ -428,7 +462,7 @@ argparse_func_with_body_ast = fix_missing_locations(
                         Store(),
                     )
                 ],
-                value=set_value(docstring_header_str),
+                value=set_value(_argparse_description_str),
                 expr=None,
                 **maybe_type_comment
             ),
@@ -500,7 +534,7 @@ argparse_func_action_append_ast = fix_missing_locations(
                 targets=[
                     Attribute(Name("argument_parser", Load()), "description", Store())
                 ],
-                value=set_value(docstring_header_str.rstrip()),
+                value=set_value(docstring_header_no_nl_str),
                 expr=None,
                 **maybe_type_comment
             ),
@@ -595,11 +629,9 @@ argparse_function_google_tf_tensorboard_ast = FunctionDef(
                 )
             ],
             lineno=None,
-            type_comment=None,
-            value=set_value(
-                deindent(tensorboard_doc_str_no_args_str, sep="  ", level=1)
-            ),
+            value=set_value(tensorboard_doc_str_no_args_str),
             expr=None,
+            **maybe_type_comment
         ),
         Expr(
             Call(
@@ -808,11 +840,11 @@ argparse_function_google_tf_tensorboard_ast = FunctionDef(
     decorator_list=[],
     name="set_cli_args",
     returns=None,
-    type_comment=None,
     arguments_args=None,
     identifier_name=None,
     stmt=None,
     lineno=None,
+    **maybe_type_comment
 )
 
 argparse_func_torch_nn_l1loss_ast = FunctionDef(
@@ -837,13 +869,13 @@ argparse_func_torch_nn_l1loss_ast = FunctionDef(
                 )
             ],
             lineno=None,
-            type_comment=None,
             value=set_value(
                 deindent(
                     remove_args_from_docstring(class_torch_nn_l1loss_docstring_str), 1
                 )
             ),
             expr=None,
+            **maybe_type_comment
         ),
         Expr(
             Call(
@@ -976,11 +1008,11 @@ argparse_func_torch_nn_l1loss_ast = FunctionDef(
     decorator_list=[],
     name="set_cli_args",
     returns=None,
-    type_comment=None,
     arguments_args=None,
     identifier_name=None,
     stmt=None,
     lineno=None,
+    **maybe_type_comment
 )
 
 argparse_add_argument_expr = Expr(
@@ -1004,7 +1036,7 @@ argparse_add_argument_expr = Expr(
 __all__ = [
     "argparse_add_argument_ast",
     "argparse_add_argument_expr",
-    "argparse_doc_str",
+    "_argparse_doc_str_tuple",
     "argparse_func_action_append_ast",
     "argparse_func_action_append_str",
     "argparse_func_ast",
@@ -1012,4 +1044,5 @@ __all__ = [
     "argparse_func_torch_nn_l1loss_ast",
     "argparse_func_with_body_ast",
     "argparse_func_with_body_str",
+    "argparse_function_google_tf_tensorboard_ast",
 ]
