@@ -1,6 +1,7 @@
 """
 Mocks for SQLalchemy
 """
+
 from ast import (
     Assign,
     Attribute,
@@ -12,16 +13,16 @@ from ast import (
     Name,
     Return,
     Store,
-    arg,
     arguments,
     keyword,
 )
 from textwrap import indent
 
-from cdd.ast_utils import maybe_type_comment, set_value
+from cdd.ast_utils import maybe_type_comment, set_arg, set_value
 from cdd.pure_utils import reindent, tab
 from cdd.tests.mocks.docstrings import (
     docstring_header_and_return_str,
+    docstring_header_and_return_two_nl_str,
     docstring_repr_str,
 )
 
@@ -30,6 +31,20 @@ _docstring_header_and_return_str = "\n{docstring}\n{tab}".format(
     tab=tab,
 )
 
+sqlalchemy_imports_str = "\n".join(
+    map(
+        "def {}(*args, **kwargs): pass\n".format,
+        (
+            "Table",
+            "Boolean",
+            "JSON",
+            "String",
+            "Column",
+            "Enum",
+        ),
+    )
+)
+
 config_tbl_str = """
 config_tbl = Table(
     "config_tbl",
@@ -59,14 +74,12 @@ config_tbl = Table(
         "as_numpy",
         Boolean,
         doc="Convert to numpy ndarrays",
-        default=None,
         nullable=True,
     ),
     Column(
         "data_loader_kwargs",
         JSON,
         doc="pass this as arguments to data_loader function",
-        default=None,
         nullable=True,
     ),
     comment={comment!r},
@@ -149,7 +162,6 @@ config_tbl_ast = Assign(
                         value=set_value("Convert to numpy ndarrays"),
                         identifier=None,
                     ),
-                    keyword(arg="default", value=set_value(None), identifier=None),
                     keyword(arg="nullable", value=set_value(True), identifier=None),
                 ],
                 expr=None,
@@ -166,7 +178,6 @@ config_tbl_ast = Assign(
                         ),
                         identifier=None,
                     ),
-                    keyword(arg="default", value=set_value(None), identifier=None),
                     keyword(arg="nullable", value=set_value(True), identifier=None),
                 ],
                 expr=None,
@@ -176,7 +187,7 @@ config_tbl_ast = Assign(
         keywords=[
             keyword(
                 arg="comment",
-                value=set_value(docstring_header_and_return_str),
+                value=set_value(docstring_header_and_return_two_nl_str),
                 identifier=None,
             )
         ],
@@ -217,14 +228,12 @@ class Config(Base):
     as_numpy = Column(
         Boolean,
         doc="Convert to numpy ndarrays",
-        default=None,
         nullable=True,
     )
 
     data_loader_kwargs = Column(
         JSON,
         doc="pass this as arguments to data_loader function",
-        default=None,
         nullable=True,
     )
 
@@ -343,7 +352,6 @@ config_decl_base_ast = ClassDef(
                         value=set_value("Convert to numpy ndarrays"),
                         identifier=None,
                     ),
-                    keyword(arg="default", value=set_value(None), identifier=None),
                     keyword(arg="nullable", value=set_value(True), identifier=None),
                 ],
                 expr=None,
@@ -366,7 +374,6 @@ config_decl_base_ast = ClassDef(
                         ),
                         identifier=None,
                     ),
-                    keyword(arg="default", value=set_value(None), identifier=None),
                     keyword(arg="nullable", value=set_value(True), identifier=None),
                 ],
                 expr=None,
@@ -381,15 +388,7 @@ config_decl_base_ast = ClassDef(
             args=arguments(
                 posonlyargs=[],
                 arg=None,
-                args=[
-                    arg(
-                        arg="self",
-                        annotation=None,
-                        expr=None,
-                        identifier_arg=None,
-                        **maybe_type_comment
-                    )
-                ],
+                args=[set_arg("self")],
                 kwonlyargs=[],
                 kw_defaults=[],
                 defaults=[],
