@@ -8,11 +8,12 @@ from copy import deepcopy
 from unittest import TestCase
 
 import cdd.emit
-import cdd.emitter_utils
-from cdd import parse
+import cdd.emit.docstring
+import cdd.emit.utils.emitter_utils
+import cdd.parse.docstring
+import cdd.parse.function
 from cdd.ast_utils import set_value
 from cdd.docstring_parsers import _set_name_and_type, parse_docstring
-from cdd.emitter_utils import to_docstring
 from cdd.tests.mocks.docstrings import (
     docstring_extra_colons_str,
     docstring_google_pytorch_lbfgs_str,
@@ -20,10 +21,12 @@ from cdd.tests.mocks.docstrings import (
     docstring_google_tf_adadelta_str,
     docstring_google_tf_adam_str,
     docstring_google_tf_lambda_callback_str,
+    docstring_google_tf_mean_squared_error_str,
     docstring_google_tf_squared_hinge_str,
     docstring_header_and_return_str,
     docstring_header_str,
     docstring_no_default_doc_str,
+    docstring_no_nl_no_none_str,
     docstring_numpydoc_only_doc_str,
     docstring_numpydoc_only_params_str,
     docstring_numpydoc_only_returns_str,
@@ -41,8 +44,10 @@ from cdd.tests.mocks.ir import (
     intermediate_repr_extra_colons,
     intermediate_repr_no_default_doc,
     intermediate_repr_no_default_doc_or_prop,
+    intermediate_repr_no_default_with_nones_doc,
     intermediate_repr_only_return_type,
 )
+from cdd.tests.mocks.methods import function_google_tf_mean_squared_error_ast
 from cdd.tests.utils_for_tests import unittest_main
 
 
@@ -94,24 +99,24 @@ class TestMarshallDocstring(TestCase):
         )
 
     def test_ir2docstring(self) -> None:
-        """ Tests whether `to_docstring` produces `docstring_str` from `intermediate_repr` """
+        """Tests whether `emit.docstring` produces `docstring_str` from `intermediate_repr`"""
         self.assertEqual(
-            docstring_str.strip(),
-            cdd.emitter_utils.to_docstring(
+            docstring_no_nl_no_none_str.strip("\n"),
+            cdd.emit.docstring.docstring(
                 deepcopy(intermediate_repr),
                 indent_level=0,
                 emit_types=True,
                 emit_default_doc=True,
                 emit_separating_tab=False,
                 word_wrap=False,
-            ).strip(),
+            ).rstrip("\n"),
         )
 
     def test_from_docstring(self) -> None:
         """
         Tests whether `docstring` produces `intermediate_repr_no_default_doc`
               from `docstring_str`"""
-        ir, returns = parse.docstring(
+        ir, returns = cdd.parse.docstring.docstring(
             docstring_str, return_tuple=True, emit_default_doc=False
         )
         self.assertTrue(returns)
@@ -121,23 +126,23 @@ class TestMarshallDocstring(TestCase):
         """
         Tests whether `docstring` produces `intermediate_repr_no_default_doc`
               from `docstring_numpydoc_str`"""
-        ir, returns = parse.docstring(
+        ir, returns = cdd.parse.docstring.docstring(
             docstring_numpydoc_str, return_tuple=True, emit_default_doc=False
         )
         self.assertTrue(returns)
-        self.assertDictEqual(ir, intermediate_repr_no_default_doc)
+        self.assertDictEqual(ir, intermediate_repr_no_default_with_nones_doc)
 
     def test_from_docstring_numpydoc_only_params(self) -> None:
         """
         Tests whether `docstring` produces `intermediate_repr_no_default_doc`
               from `docstring_numpydoc_only_params_str`"""
-        ir, returns = parse.docstring(
+        ir, returns = cdd.parse.docstring.docstring(
             docstring_numpydoc_only_params_str,
             return_tuple=True,
             emit_default_doc=False,
         )
         self.assertFalse(returns)
-        gold = deepcopy(intermediate_repr_no_default_doc)
+        gold = deepcopy(intermediate_repr_no_default_with_nones_doc)
         del gold["returns"]
         gold.update({"doc": "", "returns": None})
         self.assertDictEqual(ir, gold)
@@ -184,11 +189,16 @@ class TestMarshallDocstring(TestCase):
             ("adder", {"default": "```(5 * 5)```", "doc": "5b"}),
         )
 
+        self.assertTupleEqual(
+            _set_name_and_type(("*nums", {}), infer_type=True, word_wrap=True),
+            ("nums", {"default": tuple(), "typ": "tuple"}),
+        )
+
     def test_from_docstring_numpydoc_only_returns(self) -> None:
         """
         Tests whether `docstring` produces `intermediate_repr_no_default_doc`
               from `docstring_numpydoc_only_returns_str`"""
-        ir, returns = parse.docstring(
+        ir, returns = cdd.parse.docstring.docstring(
             docstring_numpydoc_only_returns_str,
             return_tuple=True,
             emit_default_doc=False,
@@ -209,7 +219,7 @@ class TestMarshallDocstring(TestCase):
         """
         Tests whether `docstring` produces `intermediate_repr_no_default_doc`
               from `docstring_numpydoc_only_doc_str`"""
-        ir, returns = parse.docstring(
+        ir, returns = cdd.parse.docstring.docstring(
             docstring_numpydoc_only_doc_str.strip(), return_tuple=True
         )
         self.assertFalse(returns)
@@ -230,7 +240,9 @@ class TestMarshallDocstring(TestCase):
               from `docstring_google_str`
         """
         ir = parse_docstring(docstring_google_str)
-        _intermediate_repr_no_default_doc = deepcopy(intermediate_repr_no_default_doc)
+        _intermediate_repr_no_default_doc = deepcopy(
+            intermediate_repr_no_default_with_nones_doc
+        )
         _intermediate_repr_no_default_doc["doc"] = docstring_header_str
         self.assertDictEqual(ir, _intermediate_repr_no_default_doc)
 
@@ -275,10 +287,10 @@ class TestMarshallDocstring(TestCase):
 
     def test_from_docstring_google_tf_lambda_callback_str(self) -> None:
         """
-        Tests whether `parse_docstring` produces the right IR
-              from `docstring_google_tf_lambda_callback_str`
+        Tests whether `parse_docstring` produces the same IR doc
+              as from a direct read
         """
-        self.assertDictEqual(
+        self.assertEqual(
             parse_docstring(
                 docstring_google_tf_lambda_callback_str,
                 emit_default_doc=True,
@@ -288,6 +300,59 @@ class TestMarshallDocstring(TestCase):
             docstring_google_tf_lambda_callback_ir,
         )
 
+    def test_from_docstring_google_tf_mean_squared_error_str(self) -> None:
+        """
+        Tests whether `cdd.parse.function` emits the right docstring
+              from `function_google_tf_mean_squared_error_ast`
+        """
+
+        gen = cdd.emit.docstring.docstring(
+            cdd.parse.function.function(
+                function_google_tf_mean_squared_error_ast,
+                # emit_default_doc=True,
+                infer_type=True,
+                parse_original_whitespace=True,
+            ),
+            # indent_level=0,
+            docstring_format="google",
+            emit_types=True,
+            emit_default_doc=False,
+            # emit_separating_tab=False,
+            word_wrap=False,
+        )
+
+        self.assertEqual(
+            gen,
+            docstring_google_tf_mean_squared_error_str.replace(
+                # Infer the type
+                "  weights: Optional `Tensor` whose rank is either 0, or the same rank as\n"
+                "    `labels`, and must be broadcastable to `labels` (i.e., all dimensions must\n"
+                "    be either `1`, or the same as the corresponding `losses` dimension).\n",
+                "  weights (Optional[float]): Optional `Tensor` whose rank is either 0, or the same rank as "
+                "`labels`, and must be broadcastable to `labels` (i.e., all dimensions must be either `1`, or the "
+                "same as the corresponding `losses` dimension).\n",
+                1,
+            ).replace(
+                # Word wrap everything, even if text is unchanged
+                "  Weighted loss float `Tensor`. If `reduction` is `NONE`, this has the same\n"
+                "  shape as `labels`; otherwise, it is scalar.\n",
+                "\n"
+                "   Weighted loss float `Tensor`. If `reduction` is `NONE`, this has the same shape as `labels`; "
+                "otherwise, it is scalar.\n",
+                1,
+            ),
+        )
+
+    # def test_(self):
+    #     gold_google_doc_str = """
+    #     foo
+    #
+    #     Args:
+    #       a (int):
+    #
+    #     can haz
+    #     """
+
     def test_from_docstring_google_pytorch_lbfgs_str(self) -> None:
         """
         Tests whether `parse_docstring` produces the right IR
@@ -303,19 +368,8 @@ class TestMarshallDocstring(TestCase):
             docstring_google_pytorch_lbfgs_ir,
         )
 
-    def test_to_docstring_fails(self) -> None:
-        """
-        Tests docstring failure conditions
-        """
-        self.assertRaises(
-            NotImplementedError,
-            lambda: to_docstring(
-                deepcopy(intermediate_repr_no_default_doc), docstring_format="numpydoc"
-            ),
-        )
-
     def test_docstring_header_and_return_str(self) -> None:
-        """ Tests that `docstring_header_and_return_str` can produce IR """
+        """Tests that `docstring_header_and_return_str` can produce IR"""
         _intermediate_repr = deepcopy(intermediate_repr_no_default_doc)
         _intermediate_repr["params"] = OrderedDict()
         self.assertDictEqual(
